from collections.abc import Callable, Iterable, Mapping, Sequence
from typing import Any, ClassVar, final
from typing_extensions import Self, disjoint_base

from google.protobuf.descriptor_pb2 import (
    DescriptorProto,
    EnumDescriptorProto,
    EnumOptions,
    EnumValueOptions,
    FeatureSetDefaults,
    FieldOptions,
    FileDescriptorProto,
    FileOptions,
    MessageOptions,
    MethodDescriptorProto,
    MethodOptions,
    OneofOptions,
    ServiceDescriptorProto,
    ServiceOptions,
)

default_pool: DescriptorPool

@final
class Arena: ...

@final
class Descriptor:
    containing_type: Descriptor | None
    enum_types: Sequence[EnumDescriptor]
    enum_types_by_name: Mapping[str, EnumDescriptor]
    enum_values_by_name: dict[str, EnumValueDescriptor]
    extension_ranges: list[tuple[int, int]]
    extensions: Sequence[FieldDescriptor]
    extensions_by_name: Mapping[str, FieldDescriptor]
    fields: Sequence[FieldDescriptor]
    fields_by_camelcase_name: Mapping[str, FieldDescriptor]
    fields_by_name: Mapping[str, FieldDescriptor]
    fields_by_number: Mapping[int, FieldDescriptor]
    file: FileDescriptor
    full_name: str
    has_options: bool
    is_extendable: bool
    name: str
    nested_types: Sequence[Descriptor]
    nested_types_by_name: Mapping[str, Descriptor]
    oneofs: Sequence[OneofDescriptor]
    oneofs_by_name: Mapping[str, OneofDescriptor]
    def __new__(cls, *args, **kwargs) -> Self: ...
    def CopyToProto(self, proto: DescriptorProto, /) -> None: ...
    def EnumValueName(self, enum: str, value: int) -> str: ...
    def GetOptions(self) -> MessageOptions: ...

@final
class DescriptorPool:
    def __new__(cls, *args, **kwargs) -> Self: ...
    def Add(self, file_desc_proto: FileDescriptorProto, /) -> None: ...
    def AddSerializedFile(self, serialized_file_desc_proto: bytes, /) -> FileDescriptor: ...
    def FindAllExtensions(self, message_descriptor: Descriptor, /) -> list[FieldDescriptor]: ...
    def FindEnumTypeByName(self, full_name: str, /) -> EnumDescriptor: ...
    def FindExtensionByName(self, full_name: str, /) -> FieldDescriptor: ...
    def FindExtensionByNumber(self, message_descriptor: Descriptor, number: int, /) -> FieldDescriptor: ...
    def FindFieldByName(self, full_name: str, /) -> FieldDescriptor: ...
    def FindFileByName(self, file_name: str, /) -> FileDescriptor: ...
    def FindFileContainingSymbol(self, symbol: str, /) -> FileDescriptor: ...
    def FindMessageTypeByName(self, full_name: str, /) -> Descriptor: ...
    def FindMethodByName(self, full_name: str, /) -> MethodDescriptor: ...
    def FindOneofByName(self, full_name: str, /) -> OneofDescriptor: ...
    def FindServiceByName(self, full_name: str, /) -> ServiceDescriptor: ...
    def SetFeatureSetDefaults(self, defaults: FeatureSetDefaults, /) -> None: ...

@final
class EnumDescriptor:
    containing_type: Descriptor | None
    file: FileDescriptor
    full_name: str
    has_options: bool
    is_closed: bool
    name: str
    values: Sequence[EnumValueDescriptor]
    values_by_name: Mapping[str, EnumValueDescriptor]
    values_by_number: Mapping[int, EnumValueDescriptor]
    def __new__(cls, *args, **kwargs) -> Self: ...
    def CopyToProto(self, proto: EnumDescriptorProto, /) -> None: ...
    def GetOptions(self) -> EnumOptions: ...

@final
class EnumValueDescriptor:
    has_options: bool
    index: int
    name: str
    number: int
    type: EnumDescriptor
    def __new__(cls, *args, **kwargs) -> Self: ...
    def GetOptions(self) -> EnumValueOptions: ...

@final
class ExtensionDict:
    def __contains__(self, extension_handle: FieldDescriptor, /) -> bool: ...
    def __delitem__(self, extension_handle: FieldDescriptor, /) -> None: ...
    def __eq__(self, other: object, /) -> bool: ...
    def __ge__(self, other: object, /) -> bool: ...
    def __getitem__(self, extension_handle: FieldDescriptor, /) -> Any: ...  # Any: Message, scalar, or container
    def __gt__(self, other: object, /) -> bool: ...
    def __iter__(self) -> ExtensionIterator: ...
    def __le__(self, other: object, /) -> bool: ...
    def __len__(self) -> int: ...
    def __lt__(self, other: object, /) -> bool: ...
    def __ne__(self, other: object, /) -> bool: ...
    def __setitem__(self, extension_handle: FieldDescriptor, value: Any, /) -> None: ...  # Any: Message, scalar, or container

@final
class ExtensionIterator:
    def __iter__(self) -> Self: ...
    def __next__(self) -> FieldDescriptor: ...

@final
class FieldDescriptor:
    CPPTYPE_BOOL: ClassVar[int] = ...
    CPPTYPE_BYTES: ClassVar[int] = ...
    CPPTYPE_DOUBLE: ClassVar[int] = ...
    CPPTYPE_ENUM: ClassVar[int] = ...
    CPPTYPE_FLOAT: ClassVar[int] = ...
    CPPTYPE_INT32: ClassVar[int] = ...
    CPPTYPE_INT64: ClassVar[int] = ...
    CPPTYPE_MESSAGE: ClassVar[int] = ...
    CPPTYPE_STRING: ClassVar[int] = ...
    CPPTYPE_UINT32: ClassVar[int] = ...
    CPPTYPE_UINT64: ClassVar[int] = ...
    LABEL_OPTIONAL: ClassVar[int] = ...
    LABEL_REPEATED: ClassVar[int] = ...
    LABEL_REQUIRED: ClassVar[int] = ...
    TYPE_BOOL: ClassVar[int] = ...
    TYPE_BYTES: ClassVar[int] = ...
    TYPE_DOUBLE: ClassVar[int] = ...
    TYPE_ENUM: ClassVar[int] = ...
    TYPE_FIXED32: ClassVar[int] = ...
    TYPE_FIXED64: ClassVar[int] = ...
    TYPE_FLOAT: ClassVar[int] = ...
    TYPE_GROUP: ClassVar[int] = ...
    TYPE_INT32: ClassVar[int] = ...
    TYPE_INT64: ClassVar[int] = ...
    TYPE_MESSAGE: ClassVar[int] = ...
    TYPE_SFIXED32: ClassVar[int] = ...
    TYPE_SFIXED64: ClassVar[int] = ...
    TYPE_SINT32: ClassVar[int] = ...
    TYPE_SINT64: ClassVar[int] = ...
    TYPE_STRING: ClassVar[int] = ...
    TYPE_UINT32: ClassVar[int] = ...
    TYPE_UINT64: ClassVar[int] = ...
    camelcase_name: str
    containing_oneof: OneofDescriptor | None
    containing_type: Descriptor | None
    cpp_type: int
    default_value: Any  # Any: str, int, float, bytes, or bool
    enum_type: EnumDescriptor | None
    extension_scope: Descriptor | None
    file: FileDescriptor
    full_name: str
    has_default_value: bool
    has_options: bool
    has_presence: bool
    index: int
    is_extension: bool
    is_packed: bool
    is_repeated: bool
    is_required: bool
    json_name: str
    message_type: Descriptor | None
    name: str
    number: int
    type: int
    def __new__(cls, *args, **kwargs) -> Self: ...
    def GetOptions(self) -> FieldOptions: ...

@final
class FileDescriptor:
    dependencies: Sequence[FileDescriptor]
    enum_types_by_name: Mapping[str, EnumDescriptor]
    extensions_by_name: Mapping[str, FieldDescriptor]
    has_options: bool
    message_types_by_name: Mapping[str, Descriptor]
    name: str
    package: str
    pool: DescriptorPool
    public_dependencies: Sequence[FileDescriptor]
    serialized_pb: bytes
    services_by_name: Mapping[str, ServiceDescriptor]
    def __new__(cls, *args, **kwargs) -> Self: ...
    def CopyToProto(self, proto: FileDescriptorProto, /) -> None: ...
    def GetOptions(self) -> FileOptions: ...

@final
class MapIterator:
    def __iter__(self) -> Self: ...
    def __next__(self) -> bool | int | str: ...

@final
class Message:
    Extensions: ExtensionDict
    def __init__(self, *args, **kwargs) -> None: ...
    def __contains__(self, field_name_or_key: str, /) -> bool: ...
    def __deepcopy__(self, memo: Any = None) -> Self: ...
    def __delattr__(self, name: str, /) -> None: ...
    def __eq__(self, other: object, /) -> bool: ...
    def __ge__(self, other: object, /) -> bool: ...
    def __gt__(self, other: object, /) -> bool: ...
    def __le__(self, other: object, /) -> bool: ...
    def __lt__(self, other: object, /) -> bool: ...
    def __ne__(self, other: object, /) -> bool: ...
    def __setattr__(self, name: str, value: Any, /) -> None: ...  # Any: str, int, float, bytes, bool, or Message
    def MergeFrom(self, other_msg: Message, /) -> None: ...
    def CopyFrom(self, other_msg: Message, /) -> None: ...
    def Clear(self) -> None: ...
    def SetInParent(self) -> None: ...
    def IsInitialized(self, errors: list[str] | None = None) -> bool: ...
    def MergeFromString(self, serialized: bytes, /) -> int: ...
    def ParseFromString(self, serialized: bytes, /) -> int: ...
    def SerializeToString(self, *, deterministic: bool = ...) -> bytes: ...
    def SerializePartialToString(self, *, deterministic: bool = ...) -> bytes: ...
    def ListFields(self) -> list[tuple[FieldDescriptor, Any]]: ...  # Any: str, int, float, bytes, bool, or Message
    def HasField(self, field_name: str, /) -> bool: ...
    def ClearField(self, field_name: str, /) -> None: ...
    def WhichOneof(self, oneof_group: str, /) -> str | None: ...
    def HasExtension(self, field_descriptor: FieldDescriptor, /) -> bool: ...
    def ClearExtension(self, field_descriptor: FieldDescriptor, /) -> None: ...
    def UnknownFields(self) -> UnknownFieldSet: ...
    def DiscardUnknownFields(self) -> None: ...
    def ByteSize(self) -> int: ...
    @classmethod
    def FromString(cls, s: bytes, /) -> Self: ...
    def FindInitializationErrors(self) -> list[str]: ...

@disjoint_base
class MessageMeta(type): ...

@final
class MethodDescriptor:
    client_streaming: bool
    containing_service: ServiceDescriptor
    full_name: str
    has_options: bool
    index: int
    input_type: Descriptor
    name: str
    output_type: Descriptor
    server_streaming: bool
    def __new__(cls, *args, **kwargs) -> Self: ...
    def CopyToProto(self, proto: MethodDescriptorProto, /) -> None: ...
    def GetOptions(self) -> MethodOptions: ...

@final
class OneofDescriptor:
    containing_type: Descriptor
    fields: Sequence[FieldDescriptor]
    full_name: str
    has_options: bool
    index: int
    name: str
    def __new__(cls, *args, **kwargs) -> Self: ...
    def GetOptions(self) -> OneofOptions: ...

@final
class RepeatedCompositeContainer:
    def __new__(cls, *args, **kwargs) -> Self: ...
    def MergeFrom(self, other: RepeatedCompositeContainer | Iterable[Message], /) -> None: ...
    def add(self, **kwargs: Any) -> Message: ...  # Any: field names and values
    def append(self, value: Message, /) -> None: ...
    def extend(self, values: Iterable[Message], /) -> None: ...
    def insert(self, key: int, value: Message, /) -> None: ...
    def pop(self, key: int = -1, /) -> Message: ...
    def remove(self, value: Message, /) -> None: ...
    def reverse(self) -> None: ...
    def sort(self, *, key: Callable[[Any], Any] | None = None, reverse: bool = False) -> None: ...
    def clear(self) -> None: ...
    def __deepcopy__(self, memo: Any = None) -> RepeatedCompositeContainer: ...
    def __delitem__(self, key: int | slice, /) -> None: ...
    def __eq__(self, other: object, /) -> bool: ...
    def __ge__(self, other: object, /) -> bool: ...
    def __getitem__(self, index: int | slice, /) -> Message: ...
    def __gt__(self, other: object, /) -> bool: ...
    def __le__(self, other: object, /) -> bool: ...
    def __len__(self) -> int: ...
    def __lt__(self, other: object, /) -> bool: ...
    def __ne__(self, other: object, /) -> bool: ...
    def __setitem__(self, index: int | slice, value: Message, /) -> None: ...

@final
class RepeatedScalarContainer:
    def __new__(cls, *args, **kwargs) -> Self: ...
    def MergeFrom(self, other: Any, /) -> None: ...  # Any: bool, int, float, str, or bytes
    def append(self, value: Any, /) -> None: ...
    def extend(self, values: Any, /) -> None: ...
    def insert(self, key: int, value: Any, /) -> None: ...
    def pop(self, key: int = -1, /) -> Any: ...
    def remove(self, value: Any, /) -> None: ...
    def reverse(self) -> None: ...
    def sort(self, *, key: Callable[[Any], Any] | None = None, reverse: bool = False) -> None: ...
    def clear(self) -> None: ...
    def __array__(self, dtype: Any = None, copy: bool | None = None) -> Any: ...  # Any: numpy types
    def __deepcopy__(self, memo: Any = None) -> RepeatedScalarContainer: ...
    def __delitem__(self, key: int | slice, /) -> None: ...
    def __eq__(self, other: object, /) -> bool: ...
    def __ge__(self, other: object, /) -> bool: ...
    def __getitem__(self, index: int | slice, /) -> Any: ...
    def __gt__(self, other: object, /) -> bool: ...
    def __le__(self, other: object, /) -> bool: ...
    def __len__(self) -> int: ...
    def __lt__(self, other: object, /) -> bool: ...
    def __ne__(self, other: object, /) -> bool: ...
    def __reduce__(self) -> tuple[Any, ...]: ...
    def __setitem__(self, index: int | slice, value: Any, /) -> None: ...

@final
class ServiceDescriptor:
    file: FileDescriptor
    full_name: str
    has_options: bool
    index: int
    methods: Sequence[MethodDescriptor]
    methods_by_name: Mapping[str, MethodDescriptor]
    name: str
    def __new__(cls, *args, **kwargs) -> Self: ...
    def CopyToProto(self, proto: ServiceDescriptorProto, /) -> None: ...
    def FindMethodByName(self, name: str, /) -> MethodDescriptor: ...
    def GetOptions(self) -> ServiceOptions: ...

@final
class UnknownFieldSet:
    def __new__(cls, *args, **kwargs) -> Self: ...
    def __getitem__(self, index: int, /) -> Any: ...  # Any: internal unknown field object
    def __len__(self) -> int: ...

def SetAllowOversizeProtos(allow: bool, /) -> None: ...



from braintree.resource import Resource
from braintree.subscription import Subscription

class VenmoAccount(Resource):
    subscriptions: list[Subscription]
    def __init__(self, gateway, attributes) -> None: ...



import builtins
from _typeshed import Incomplete, Unused
from collections.abc import Callable, Collection
from typing import Any, ClassVar, Final, Literal, NoReturn, SupportsIndex, TypeAlias, TypeVar, overload
from typing_extensions import Self, deprecated

import numpy as np
import pandas as pd
from numpy.typing import ArrayLike, DTypeLike, NDArray
from pandas._typing import ScalarIndexer, SequenceIndexer, TakeIndexer
from pandas.api.extensions import ExtensionArray, ExtensionDtype
from pyproj import CRS, Transformer
from shapely import Geometry
from shapely.geometry.base import BaseGeometry

from .base import _AffinityOrigin, _ConvertibleToCRS
from .sindex import SpatialIndex

_ScalarType = TypeVar("_ScalarType", bound=np.generic)
_Array1D: TypeAlias = np.ndarray[tuple[int], np.dtype[_ScalarType]]
_Array2D: TypeAlias = np.ndarray[tuple[int, int], np.dtype[_ScalarType]]
_ArrayOrGeom: TypeAlias = GeometryArray | ArrayLike | Geometry

TransformerFromCRS = Transformer.from_crs
POLYGON_GEOM_TYPES: Final[set[str]]
LINE_GEOM_TYPES: Final[set[str]]
POINT_GEOM_TYPES: Final[set[str]]

class GeometryDtype(ExtensionDtype):
    type: ClassVar[type[BaseGeometry]]
    name: ClassVar[str]
    na_value: None
    @classmethod
    def construct_from_string(cls, string: str) -> Self: ...
    @classmethod
    def construct_array_type(cls) -> builtins.type[GeometryArray]: ...

def isna(value: object) -> bool: ...
def from_shapely(data, crs: _ConvertibleToCRS | None = None) -> GeometryArray: ...
def to_shapely(geoms: GeometryArray) -> _Array1D[np.object_]: ...
def from_wkb(
    data, crs: _ConvertibleToCRS | None = None, on_invalid: Literal["raise", "warn", "ignore", "fix"] = "raise"
) -> GeometryArray: ...

@overload
def to_wkb(geoms: GeometryArray, hex: Literal[False] = False, **kwargs) -> _Array1D[np.bytes_]: ...
@overload
def to_wkb(geoms: GeometryArray, hex: Literal[True], **kwargs) -> _Array1D[np.str_]: ...

def from_wkt(
    data, crs: _ConvertibleToCRS | None = None, on_invalid: Literal["raise", "warn", "ignore", "fix"] = "raise"
) -> GeometryArray: ...
def to_wkt(geoms: GeometryArray, **kwargs) -> _Array1D[np.str_]: ...
def points_from_xy(
    x: ArrayLike, y: ArrayLike, z: ArrayLike | None = None, crs: _ConvertibleToCRS | None = None
) -> GeometryArray: ...

class GeometryArray(ExtensionArray):
    def __init__(self, data: GeometryArray | NDArray[np.object_], crs: _ConvertibleToCRS | None = None) -> None: ...
    @property
    def sindex(self) -> SpatialIndex: ...
    @property
    def has_sindex(self) -> bool: ...

    @property
    def crs(self) -> CRS | None: ...
    @crs.setter
    def crs(self, value: _ConvertibleToCRS | None) -> None: ...

    def check_geographic_crs(self, stacklevel: int) -> None: ...
    @property
    def dtype(self) -> GeometryDtype: ...
    def __len__(self) -> int: ...

    # np.integer[Any] because precision is not important
    @overload
    def __getitem__(self, idx: ScalarIndexer) -> BaseGeometry: ...  # Always 1-D, doesn't accept tuple
    @overload
    def __getitem__(self, idx: SequenceIndexer) -> GeometryArray: ...

    def __setitem__(
        self, key, value: _ArrayOrGeom | pd.DataFrame | pd.Series[Any]  # Cannot use pd.Series[BaseGeometry]
    ) -> None: ...
    @property
    def is_valid(self) -> _Array1D[np.bool_]: ...
    def is_valid_reason(self) -> _Array1D[np.object_]: ...
    def is_valid_coverage(self, gap_width: float = 0.0) -> bool: ...
    def invalid_coverage_edges(self, gap_width: float = 0.0) -> _Array1D[np.object_]: ...
    @property
    def is_empty(self) -> _Array1D[np.bool_]: ...
    @property
    def is_simple(self) -> _Array1D[np.bool_]: ...
    @property
    def is_ring(self) -> _Array1D[np.bool_]: ...
    @property
    def is_closed(self) -> _Array1D[np.bool_]: ...
    @property
    def is_ccw(self) -> _Array1D[np.bool_]: ...
    @property
    def has_z(self) -> _Array1D[np.bool_]: ...
    @property
    def has_m(self) -> _Array1D[np.bool_]: ...
    @property
    def geom_type(self) -> _Array1D[np.str_]: ...
    @property
    def area(self) -> _Array1D[np.float64]: ...
    @property
    def length(self) -> _Array1D[np.float64]: ...
    def count_coordinates(self) -> _Array1D[np.int32]: ...
    def count_geometries(self) -> _Array1D[np.int32]: ...
    def count_interior_rings(self) -> _Array1D[np.int32]: ...
    def get_precision(self) -> _Array1D[np.float64]: ...
    def get_geometry(self, index: SupportsIndex | ArrayLike) -> _Array1D[np.object_]: ...
    @property
    def boundary(self) -> GeometryArray: ...
    @property
    def centroid(self) -> GeometryArray: ...
    def concave_hull(self, ratio: float, allow_holes: bool) -> _Array1D[np.object_]: ...
    def constrained_delaunay_triangles(self) -> GeometryArray: ...
    @property
    def convex_hull(self) -> GeometryArray: ...
    @property
    def envelope(self) -> GeometryArray: ...
    def minimum_rotated_rectangle(self) -> GeometryArray: ...
    @property
    def exterior(self) -> GeometryArray: ...
    def extract_unique_points(self) -> GeometryArray: ...
    def offset_curve(
        self,
        distance: float | ArrayLike,
        quad_segs: int = 8,
        join_style: Literal["round", "bevel", "mitre"] = "round",
        mitre_limit: float = 5.0,
    ) -> GeometryArray: ...
    @property
    def interiors(self) -> _Array1D[np.object_]: ...
    def remove_repeated_points(self, tolerance: float | ArrayLike = 0.0) -> GeometryArray: ...
    def representative_point(self) -> GeometryArray: ...
    def minimum_bounding_circle(self) -> GeometryArray: ...
    def maximum_inscribed_circle(self, tolerance: float | ArrayLike) -> GeometryArray: ...
    def minimum_bounding_radius(self) -> _Array1D[np.float64]: ...
    def minimum_clearance(self) -> _Array1D[np.float64]: ...
    def minimum_clearance_line(self) -> GeometryArray: ...
    def normalize(self) -> GeometryArray: ...
    def orient_polygons(self, exterior_cw: bool = False) -> GeometryArray: ...
    def make_valid(self, method: Literal["linework", "structure"] = "linework", keep_collapsed: bool = True) -> GeometryArray: ...
    def reverse(self) -> GeometryArray: ...
    def segmentize(self, max_segment_length: float | ArrayLike) -> GeometryArray: ...
    def force_2d(self) -> GeometryArray: ...
    def force_3d(self, z: float | ArrayLike = 0) -> GeometryArray: ...
    def transform(
        self, transformation: Callable[[NDArray[np.float64]], NDArray[np.float64]], include_z: bool = False
    ) -> GeometryArray: ...
    def line_merge(self, directed: bool = False) -> GeometryArray: ...
    def set_precision(
        self, grid_size: float, mode: Literal["valid_output", "pointwise", "keep_collapsed", 0, 1, 2] = "valid_output"
    ) -> GeometryArray: ...
    def covers(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def covered_by(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def contains(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def contains_properly(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def crosses(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def disjoint(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def geom_equals(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def intersects(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def overlaps(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def touches(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def within(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def dwithin(self, other: _ArrayOrGeom, distance: float) -> _Array1D[np.bool_]: ...
    def geom_equals_exact(self, other: _ArrayOrGeom, tolerance: float | ArrayLike) -> _Array1D[np.bool_]: ...
    def geom_equals_identical(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def clip_by_rect(self, xmin: float, ymin: float, xmax: float, ymax: float) -> GeometryArray: ...
    def difference(self, other: _ArrayOrGeom) -> GeometryArray: ...
    def intersection(self, other: _ArrayOrGeom) -> GeometryArray: ...
    def symmetric_difference(self, other: _ArrayOrGeom) -> GeometryArray: ...
    def union(self, other: _ArrayOrGeom) -> GeometryArray: ...
    def shortest_line(self, other: _ArrayOrGeom) -> GeometryArray: ...
    def snap(self, other: _ArrayOrGeom, tolerance: float | ArrayLike) -> GeometryArray: ...
    def shared_paths(self, other: _ArrayOrGeom) -> GeometryArray: ...
    def distance(self, other: _ArrayOrGeom) -> _Array1D[np.float64]: ...
    def hausdorff_distance(self, other: _ArrayOrGeom, **kwargs) -> _Array1D[np.float64]: ...
    def frechet_distance(self, other: _ArrayOrGeom, **kwargs) -> _Array1D[np.float64]: ...
    def buffer(self, distance: float | ArrayLike, resolution: int = 16, **kwargs) -> GeometryArray: ...
    def interpolate(self, distance: float | ArrayLike, normalized: bool = False) -> GeometryArray: ...
    def simplify(self, tolerance: float | ArrayLike, preserve_topology: bool = True) -> GeometryArray: ...
    def simplify_coverage(self, tolerance: float | ArrayLike, simplify_boundary: bool = True) -> GeometryArray: ...
    def project(self, other: _ArrayOrGeom, normalized: bool = False) -> _Array1D[np.float64]: ...
    def relate(self, other: _ArrayOrGeom) -> _Array1D[np.str_]: ...
    def relate_pattern(self, other: _ArrayOrGeom, pattern: str) -> _Array1D[np.bool_]: ...
    @deprecated("Use method `union_all` instead.")
    def unary_union(self) -> BaseGeometry: ...
    def union_all(
        self, method: Literal["coverage", "unary", "disjoint_subset"] = "unary", grid_size: float | None = None
    ) -> BaseGeometry: ...
    def intersection_all(self) -> BaseGeometry: ...
    def affine_transform(self, matrix: Collection[float]) -> GeometryArray: ...
    def translate(self, xoff: float = 0.0, yoff: float = 0.0, zoff: float = 0.0) -> GeometryArray: ...
    def rotate(self, angle: float, origin: _AffinityOrigin = "center", use_radians: bool = False) -> GeometryArray: ...
    def scale(
        self, xfact: float = 1.0, yfact: float = 1.0, zfact: float = 1.0, origin: _AffinityOrigin = "center"
    ) -> GeometryArray: ...
    def skew(
        self, xs: float = 0.0, ys: float = 0.0, origin: _AffinityOrigin = "center", use_radians: bool = False
    ) -> GeometryArray: ...
    def to_crs(self, crs: _ConvertibleToCRS | None = None, epsg: int | None = None) -> GeometryArray: ...
    def estimate_utm_crs(self, datum_name: str = "WGS 84") -> CRS: ...
    @property
    def x(self) -> _Array1D[np.float64]: ...
    @property
    def y(self) -> _Array1D[np.float64]: ...
    @property
    def z(self) -> _Array1D[np.float64]: ...
    @property
    def m(self) -> _Array1D[np.float64]: ...
    @property
    def bounds(self) -> _Array2D[np.float64]: ...
    @property
    def total_bounds(self) -> _Array1D[np.float64]: ...
    @property
    def size(self) -> int: ...
    @property
    def shape(self) -> tuple[int]: ...  # Always 1-D, this is not mistaken for tuple[int, ...]
    @property
    def ndim(self) -> Literal[1]: ...
    def copy(self, *args: Unused, **kwargs: Unused) -> GeometryArray: ...
    def take(self, indices: TakeIndexer, allow_fill: bool = False, fill_value: Geometry | None = None) -> GeometryArray: ...
    def fillna(  # type: ignore[override]
        self,
        value: Geometry | GeometryArray | None = None,
        method: Literal["backfill", "bfill", "pad", "ffill"] | None = None,
        limit: int | None = None,
        copy: bool = True,
    ) -> GeometryArray: ...

    @overload  # type: ignore[override]
    def astype(self, dtype: GeometryDtype, copy: bool = True) -> GeometryArray: ...
    @overload
    def astype(self, dtype: ExtensionDtype | Literal["string"], copy: bool = True) -> ExtensionArray: ...  # type: ignore[overload-overlap]
    @overload
    def astype(self, dtype: DTypeLike, copy: bool = True) -> _Array1D[Incomplete]: ...

    def isna(self) -> _Array1D[np.bool_]: ...
    def value_counts(self, dropna: bool = True) -> pd.Series[int]: ...
    def unique(self) -> GeometryArray: ...
    @property
    def nbytes(self) -> int: ...
    def shift(self, periods: int = 1, fill_value: Geometry | None = None) -> GeometryArray: ...  # type: ignore[override]
    def argmin(self, skipna: bool = True) -> NoReturn: ...
    def argmax(self, skipna: bool = True) -> NoReturn: ...
    def __array__(self, dtype: DTypeLike | None = None, copy: bool | None = None) -> _Array1D[np.object_]: ...
    def __eq__(self, other: object) -> _Array1D[np.bool_]: ...  # type: ignore[override]
    def __ne__(self, other: object) -> _Array1D[np.bool_]: ...  # type: ignore[override]
    def __contains__(self, item: object) -> np.bool_: ...

# TODO: Improve `func` type with a callable protocol (with overloads for 2D and 3D geometries)
def transform(
    data: NDArray[np.object_] | GeometryArray | pd.Series[Any],  # Cannot use pd.Series[BaseGeometry]
    func: Callable[..., NDArray[np.float64]],
) -> NDArray[np.object_]: ...



import re
import unittest
from _typeshed import Incomplete, StrPath, Unused
from configparser import ConfigParser, _SectionName
from types import ModuleType
from typing import Final, Literal

__version__: Final[str]

def invariantSeed(n: float | str | bytes | bytearray | None) -> None: ...
def haveRenderPM() -> ModuleType | Literal[False]: ...

DEJAVUSANS: Final = ("DejaVuSans", "DejaVuSans-Bold", "DejaVuSans-Oblique", "DejaVuSans-BoldOblique")

def haveDejaVu() -> bool: ...
def isWritable(D: Unused) -> Literal[0, 1]: ...

RL_HOME: str | None
testsFolder: str | None

def setOutDir(name: str) -> str: ...
def mockUrlRead(name: str): ...
def outputfile(fn: StrPath | None) -> str: ...
def printLocation(depth: int = 1) -> None: ...
def makeSuiteForClasses(*classes: type[unittest.TestCase], testMethodPrefix: str | None = None) -> unittest.TestSuite: ...
def getCVSEntries(folder: StrPath, files: bool | Literal[1, 0] = 1, folders: bool | Literal[1, 0] = 0) -> list[str]: ...

class ExtConfigParser(ConfigParser):
    pat: re.Pattern[str]
    def getstringlist(self, section: _SectionName, option: str): ...

class GlobDirectoryWalker:
    index: int
    pattern: str
    stack: list[str]
    files: list[str]
    directory: str
    def __init__(self, directory: str, pattern: str = "*") -> None: ...
    def __getitem__(self, index) -> str | None: ...
    def filterFiles(self, folder, files): ...

class RestrictedGlobDirectoryWalker(GlobDirectoryWalker):
    ignorePatterns: Incomplete
    def __init__(self, directory, pattern: str = "*", ignore=None) -> None: ...
    def filterFiles(self, folder, files): ...

class CVSGlobDirectoryWalker(GlobDirectoryWalker):
    def filterFiles(self, folder, files): ...

class SecureTestCase(unittest.TestCase):
    def setUp(self) -> None: ...
    def tearDown(self) -> None: ...

class NearTestCase(unittest.TestCase):
    @staticmethod
    def assertNear(a, b, accuracy: float = 1e-05) -> None: ...

class ScriptThatMakesFileTest(unittest.TestCase):
    scriptDir: Incomplete
    scriptName: Incomplete
    outFileName: Incomplete
    verbose: Incomplete
    def __init__(self, scriptDir, scriptName, outFileName, verbose: int = 0) -> None: ...
    cwd: Incomplete
    def setUp(self) -> None: ...
    def tearDown(self) -> None: ...
    def runTest(self) -> None: ...

def equalStrings(a: str | bytes, b: str | bytes, enc: str = "utf8") -> bool: ...
def eqCheck(r, x) -> None: ...
def rlextraNeeded() -> bool: ...
def rlSkipIf(cond, reason, __module__=None): ...
def rlSkipUnless(cond, reason, __module__=None): ...
def rlSkip(reason, __module__=None): ...



from dateparser.conf import Settings
from dateparser.languages.locale import Locale
from dateparser.search.detection import BaseLanguageDetector

class FullTextLanguageDetector(BaseLanguageDetector):
    language_unique_chars: list[set[str]]
    language_chars: list[set[str]]
    def __init__(self, languages: list[Locale]) -> None: ...
    def get_unique_characters(self, settings: Settings) -> None: ...
    def character_check(self, date_string: str, settings: Settings) -> None: ...



from _typeshed import StrPath
from collections.abc import Iterable
from typing import ClassVar, Literal

from . import unix

class Compiler(unix.Compiler):
    src_extensions: ClassVar[list[str]]
    zos_compiler: Literal["ibm-openxl", "ibm-xlclang", "ibm-xlc"]
    def __init__(self, verbose: bool = False, force: bool = False) -> None: ...
    def runtime_library_dir_option(self, dir: str) -> str: ...
    def link(
        self,
        target_desc: str,
        objects: list[str] | tuple[str, ...],
        output_filename: str,
        output_dir: str | None = None,
        libraries: list[str] | tuple[str, ...] | None = None,
        library_dirs: list[str] | tuple[str, ...] | None = None,
        runtime_library_dirs: list[str] | tuple[str, ...] | None = None,
        export_symbols: Iterable[str] | None = None,
        debug: bool = False,
        extra_preargs: list[str] | None = None,
        extra_postargs: list[str] | None = None,
        build_temp: StrPath | None = None,
        target_lang: str | None = None,
    ) -> None: ...



from collections.abc import Iterable, Iterator
from typing import Literal

from .ast import Node

class Color:
    COLOR_SPACES: set[str] | None
    def __init__(self, space: str, coordinates: tuple[float | None, ...], alpha: float) -> None: ...
    def __iter__(self) -> Iterator[float | None]: ...
    def __getitem__(self, key: int) -> float: ...
    def __hash__(self) -> int: ...
    def __eq__(self, other: object) -> bool: ...
    def to(self, space: str) -> Color: ...

def parse_color(input: str | Iterable[Node]) -> Color | Literal["currentcolor"] | None: ...

COLOR_SPACES: set[str]
D50: tuple[float, float, float]
D65: tuple[float, float, float]



from _typeshed import Incomplete
from typing import Final

TYPE_NONE: Final = 0x00
TYPE_BOOL: Final = 0x01
TYPE_INT64: Final = 0x05
TYPE_FLOAT64: Final = 0x06
TYPE_BYTES: Final = 0x10
TYPE_STRING: Final = 0x11
TYPE_LIST: Final = 0x20
TYPE_DICT: Final = 0x21
MAX_STRING_SIZE: Final = 67108864
MAX_BYTES_SIZE: Final = 67108864
MAX_LIST_SIZE: Final = 1048576
MAX_DICT_SIZE: Final = 1048576

class TLVEncoder:
    @staticmethod
    def encode(
        value: bool | float | str | bytes | list[Incomplete] | tuple[Incomplete, ...] | dict[object, Incomplete] | None,
    ) -> bytes: ...  # dict key passed to `str()` function
    @staticmethod
    def decode(data: bytes, offset: int = 0) -> tuple[Incomplete, int]: ...
    @staticmethod
    def decode_full(data: bytes): ...



from _typeshed import Incomplete

from authlib.oauth2.rfc7521 import AssertionClient

from .oauth2_session import OAuth2Auth

class AssertionAuth(OAuth2Auth):
    def ensure_active_token(self): ...

# Inherits from requests.Session
class AssertionSession(AssertionClient):
    token_auth_class = AssertionAuth
    JWT_BEARER_GRANT_TYPE: Incomplete
    ASSERTION_METHODS: Incomplete
    DEFAULT_GRANT_TYPE: Incomplete
    default_timeout: Incomplete
    def __init__(
        self,
        token_endpoint,
        issuer,
        subject,
        audience=None,
        grant_type=None,
        claims=None,
        token_placement="header",
        scope=None,
        default_timeout=None,
        leeway=60,
        **kwargs,
    ) -> None: ...
    def request(self, method, url, withhold_token=False, auth=None, **kwargs): ...



from _typeshed import Incomplete
from collections.abc import Iterable
from ssl import SSLSocket
from typing import Any, ClassVar, TypeAlias

from gunicorn.config import Config
from gunicorn.http2.request import HTTP2Request
from gunicorn.http2.stream import HTTP2Stream

from .._types import _AddressType

_H2Connection: TypeAlias = Any  # h2.connection.H2Connection class

class HTTP2ServerConnection:
    READ_BUFFER_SIZE: ClassVar[int]
    cfg: Config
    sock: SSLSocket
    client_addr: _AddressType
    streams: dict[int, HTTP2Stream]
    initial_window_size: int
    max_concurrent_streams: int
    max_frame_size: int
    max_header_list_size: int
    h2_conn: _H2Connection

    def __init__(self, cfg: Config, sock: SSLSocket, client_addr: _AddressType) -> None: ...
    def initiate_connection(self) -> None: ...
    def receive_data(self, data: bytes | None = None) -> list[HTTP2Request]: ...
    def send_informational(self, stream_id: int, status: int, headers: Iterable[tuple[str, Incomplete]]) -> None: ...
    def send_response(
        self, stream_id: int, status: int, headers: Iterable[tuple[str, Incomplete]], body: bytes | None = None
    ) -> bool: ...
    def send_data(self, stream_id: int, data: bytes, end_stream: bool = False) -> bool: ...
    def send_trailers(self, stream_id: int, trailers: Iterable[tuple[str, Incomplete]]) -> bool: ...
    def send_error(self, stream_id: int, status_code: int, message: str | None = None) -> None: ...
    def reset_stream(self, stream_id: int, error_code: int = 0x8) -> None: ...
    def close(self, error_code: int = 0x0, last_stream_id: int | None = None) -> None: ...
    @property
    def is_closed(self) -> bool: ...
    def cleanup_stream(self, stream_id: int) -> None: ...

__all__ = ["HTTP2ServerConnection"]



from collections.abc import Callable

__all__ = ["Sequencer", "getSequencer", "setSequencer"]

from typing import overload

class _Counter:
    _value: int
    def __init__(self) -> None: ...
    def setFormatter(self, formatFunc: Callable[[int], str]) -> None: ...
    def reset(self, value: int | None = None) -> None: ...
    def next(self) -> int: ...
    __next__ = next
    def _this(self) -> int: ...
    def nextf(self) -> str: ...
    def thisf(self) -> str: ...
    def chain(self, otherCounter: _Counter) -> None: ...

class Sequencer:
    def __init__(self) -> None: ...
    def __next__(self) -> int: ...
    def next(self, counter=None) -> int: ...
    def thisf(self, counter=None) -> str: ...
    def nextf(self, counter=None) -> str: ...
    def setDefaultCounter(self, default=None) -> None: ...
    def registerFormat(self, format: str, func: Callable[[int], str]) -> None: ...
    def setFormat(self, counter, format: str) -> None: ...
    def reset(self, counter=None, base: int = 0) -> None: ...
    def chain(self, parent, child) -> None: ...
    def __getitem__(self, key: str) -> str: ...
    def format(self, template: str) -> str: ...
    def dump(self) -> None: ...

def getSequencer() -> Sequencer: ...

@overload
def setSequencer(seq: Sequencer) -> Sequencer: ...
@overload
def setSequencer(seq: None) -> None: ...



import socket
from collections.abc import Iterable
from typing import Literal, overload

from gunicorn.config import Config
from gunicorn.http.message import Message as Message, Request as Request
from gunicorn.http.parser import RequestParser as RequestParser
from gunicorn.http2.connection import HTTP2ServerConnection
from gunicorn.uwsgi.parser import UWSGIParser

from .._types import _AddressType

@overload
def get_parser(
    cfg: Config,
    source: socket.socket | Iterable[bytes],
    source_addr: _AddressType,
    http2_connection: Literal[False] | None = False,
) -> UWSGIParser | RequestParser: ...
@overload
def get_parser(
    cfg: Config, source: socket.socket | Iterable[bytes], source_addr: _AddressType, http2_connection: Literal[True] = ...
) -> HTTP2ServerConnection: ...

__all__ = ["Message", "Request", "RequestParser", "get_parser"]



from gunicorn.arbiter import Arbiter
from gunicorn.ctl.handlers import CommandHandlers

class ControlSocketServer:
    arbiter: Arbiter
    socket_path: str
    socket_mode: int
    handlers: CommandHandlers
    def __init__(self, arbiter: Arbiter, socket_path: str, socket_mode: int = 0o600) -> None: ...
    def start(self) -> None: ...
    def stop(self) -> None: ...



import sys
from collections.abc import Iterable
from types import GenericAlias
from typing import Any, Generic, TypeVar, overload
from typing_extensions import Self

__all__ = ["SharedMemory", "ShareableList"]

_SLT = TypeVar("_SLT", int, float, bool, str, bytes, None)

class SharedMemory:
    if sys.version_info >= (3, 13):
        def __init__(self, name: str | None = None, create: bool = False, size: int = 0, *, track: bool = True) -> None: ...
    else:
        def __init__(self, name: str | None = None, create: bool = False, size: int = 0) -> None: ...

    @property
    def buf(self) -> memoryview | None: ...
    @property
    def name(self) -> str: ...
    @property
    def size(self) -> int: ...
    def close(self) -> None: ...
    def unlink(self) -> None: ...
    def __del__(self) -> None: ...

class ShareableList(Generic[_SLT]):
    shm: SharedMemory

    @overload
    def __init__(self, sequence: None = None, *, name: str | None = None) -> None: ...
    @overload
    def __init__(self, sequence: Iterable[_SLT], *, name: str | None = None) -> None: ...

    def __getitem__(self, position: int) -> _SLT: ...
    def __setitem__(self, position: int, value: _SLT) -> None: ...
    def __reduce__(self) -> tuple[Self, tuple[_SLT, ...]]: ...
    def __len__(self) -> int: ...
    @property
    def format(self) -> str: ...
    def count(self, value: _SLT) -> int: ...
    def index(self, value: _SLT) -> int: ...
    def __class_getitem__(cls, item: Any, /) -> GenericAlias: ...



class UWSGIParseException(Exception): ...

class InvalidUWSGIHeader(UWSGIParseException):
    msg: str
    code: int

    def __init__(self, msg: str = "") -> None: ...

class UnsupportedModifier(UWSGIParseException):
    modifier: int
    code: int

    def __init__(self, modifier: int) -> None: ...

class ForbiddenUWSGIRequest(UWSGIParseException):
    host: str
    code: int

    def __init__(self, host: str) -> None: ...



import re
from datetime import datetime
from time import struct_time
from typing import Final, Protocol, type_check_only

TIME_MATCHER: Final[re.Pattern[str]]
MS_SEARCHER: Final[re.Pattern[str]]

@type_check_only
class _strptime_time(Protocol):
    def __call__(self, data_string: str, format: str = "%a %b %d %H:%M:%S %Y") -> struct_time: ...

def patch_strptime() -> _strptime_time: ...
def strptime(date_string: str, format: str) -> datetime: ...



from _typeshed import Incomplete
from typing import Final
from typing_extensions import LiteralString

__version__: Final[str]

def asNative(s) -> str: ...

EXTENSIONS: Final = [".ttf", ".ttc", ".otf", ".pfb", ".pfa"]
FF_FIXED: Final = 1
FF_SERIF: Final = 2
FF_SYMBOLIC: Final = 4
FF_SCRIPT: Final = 8
FF_NONSYMBOLIC: Final = 32
FF_ITALIC: Final = 64
FF_ALLCAP: Final = 65536
FF_SMALLCAP: Final = 131072
FF_FORCEBOLD: Final = 262144

class FontDescriptor:
    name: Incomplete
    fullName: Incomplete
    familyName: Incomplete
    styleName: Incomplete
    isBold: bool
    isItalic: bool
    isFixedPitch: bool
    isSymbolic: bool
    typeCode: Incomplete
    fileName: Incomplete
    metricsFileName: Incomplete
    timeModified: int
    def __init__(self) -> None: ...
    def getTag(self) -> LiteralString: ...

class FontFinder:
    useCache: Incomplete
    validate: Incomplete
    verbose: Incomplete
    def __init__(
        self, dirs=[], useCache: bool = True, validate: bool = False, recur: bool = False, fsEncoding=None, verbose: int = 0
    ) -> None: ...
    def addDirectory(self, dirName, recur=None) -> None: ...
    def addDirectories(self, dirNames, recur=None) -> None: ...
    def getFamilyNames(self) -> list[bytes]: ...
    def getFontsInFamily(self, familyName): ...
    def getFamilyXmlReport(self) -> LiteralString: ...
    def getFontsWithAttributes(self, **kwds) -> list[FontDescriptor]: ...
    def getFont(self, familyName, bold: bool = False, italic: bool = False) -> FontDescriptor: ...
    def save(self, fileName) -> None: ...
    def load(self, fileName) -> None: ...
    def search(self) -> None: ...

def test() -> None: ...



import datetime
from _typeshed import Incomplete, SupportsItems
from collections.abc import Callable, Iterable
from typing import Any, ClassVar, Final, Literal, NamedTuple, TypeVar, overload
from typing_extensions import Self

from .alarms import Alarms
from .caselessdict import CaselessDict
from .error import IncompleteComponent as IncompleteComponent
from .parser import Contentline, Contentlines
from .parser_tools import ICAL_TYPE
from .prop import TypesFactory, _vType, vRecur
from .timezone.tzp import TZP

_D = TypeVar("_D")

__all__ = [
    "Alarm",
    "Calendar",
    "Component",
    "ComponentFactory",
    "Event",
    "FreeBusy",
    "INLINE",
    "Journal",
    "Timezone",
    "TimezoneDaylight",
    "TimezoneStandard",
    "Todo",
    "component_factory",
    "get_example",
    "IncompleteComponent",
]

def get_example(component_directory: str, example_name: str) -> bytes: ...

class ComponentFactory(CaselessDict[Incomplete]):
    # Inherit complex __init__ from CaselessDict<-dict.
    ...

INLINE: CaselessDict[int]

class Component(CaselessDict[Incomplete]):
    name: ClassVar[str | None]
    required: ClassVar[tuple[str, ...]]
    singletons: ClassVar[tuple[str, ...]]
    multiple: ClassVar[tuple[str, ...]]
    exclusive: ClassVar[tuple[str, ...]]
    inclusive: ClassVar[tuple[tuple[str, ...], ...]]
    ignore_exceptions: ClassVar[bool]
    subcomponents: list[Incomplete]
    errors: list[str]

    # Inherit complex __init__ from CaselessDict<-dict.
    def __bool__(self) -> bool: ...
    __nonzero__ = __bool__
    def is_empty(self) -> bool: ...

    @overload
    def add(self, name: str, value: Any, *, encode: Literal[False]) -> None: ...
    @overload
    def add(self, name: str, value: Any, parameters: None, encode: Literal[False]) -> None: ...
    @overload
    def add(
        self, name: str, value: Any, parameters: SupportsItems[str, str | None] | None = None, encode: Literal[True] = True
    ) -> None: ...

    def decoded(self, name: str, default: _D = ...) -> Incomplete | _D: ...
    def get_inline(self, name: str, decode: bool = True) -> list[Incomplete]: ...

    @overload
    def set_inline(self, name: str, values: Iterable[str], encode: Literal[False] = ...) -> None: ...
    @overload
    def set_inline(self, name: str, values: Iterable[Incomplete], encode: Literal[True] = True) -> None: ...

    def add_component(self, component: Component) -> None: ...
    def walk(self, name: str | None = None, select: Callable[[Component], bool] = ...) -> list[Component]: ...
    def property_items(self, recursive: bool = True, sorted: bool = True) -> list[tuple[str, object]]: ...

    @overload
    @classmethod
    def from_ical(cls, st: str, multiple: Literal[False] = False) -> Component: ...  # or any of its subclasses
    @overload
    @classmethod
    def from_ical(cls, st: str, multiple: Literal[True]) -> list[Component]: ...  # or any of its subclasses

    def content_line(self, name: str, value: _vType | ICAL_TYPE, sorted: bool = True) -> Contentline: ...
    def content_lines(self, sorted: bool = True) -> Contentlines: ...
    def to_ical(self, sorted: bool = True) -> bytes: ...
    def __eq__(self, other: Component) -> bool: ...  # type: ignore[override]

    @property
    def DTSTAMP(self) -> datetime.datetime | None: ...
    @DTSTAMP.setter
    def DTSTAMP(self, value: datetime.datetime) -> None: ...
    @DTSTAMP.deleter
    def DTSTAMP(self) -> None: ...

    @property
    def LAST_MODIFIED(self) -> datetime.datetime | None: ...
    @LAST_MODIFIED.setter
    def LAST_MODIFIED(self, value: datetime.datetime) -> None: ...
    @LAST_MODIFIED.deleter
    def LAST_MODIFIED(self) -> None: ...

    def is_thunderbird(self) -> bool: ...

# type_def is a TypeForm
def create_single_property(
    prop: str, value_attr: str | None, value_type: tuple[type, ...], type_def: Any, doc: str, vProp: type[Incomplete] = ...
) -> property: ...

class Event(Component):
    name: ClassVar[Literal["VEVENT"]]
    @property
    def alarms(self) -> Alarms: ...
    @classmethod
    def example(cls, name: str = "rfc_9074_example_3") -> Event: ...

    @property
    def DTSTART(self) -> datetime.date | datetime.datetime | None: ...
    @DTSTART.setter
    def DTSTART(self, value: datetime.date | datetime.datetime | None) -> None: ...
    @DTSTART.deleter
    def DTSTART(self) -> None: ...

    @property
    def DTEND(self) -> datetime.date | datetime.datetime | None: ...
    @DTEND.setter
    def DTEND(self, value: datetime.date | datetime.datetime | None) -> None: ...
    @DTEND.deleter
    def DTEND(self) -> None: ...

    @property
    def DURATION(self) -> datetime.timedelta | None: ...
    @DURATION.setter
    def DURATION(self, value: datetime.timedelta | None) -> None: ...
    @DURATION.deleter
    def DURATION(self) -> None: ...

    @property
    def duration(self) -> datetime.timedelta: ...

    @property
    def start(self) -> datetime.date | datetime.datetime: ...
    @start.setter
    def start(self, value: datetime.date | datetime.datetime | None) -> None: ...

    @property
    def end(self) -> datetime.date | datetime.datetime: ...
    @end.setter
    def end(self, value: datetime.date | datetime.datetime | None) -> None: ...

    @property
    def X_MOZ_SNOOZE_TIME(self) -> datetime.datetime | None: ...
    @X_MOZ_SNOOZE_TIME.setter
    def X_MOZ_SNOOZE_TIME(self, value: datetime.datetime) -> None: ...
    @X_MOZ_SNOOZE_TIME.deleter
    def X_MOZ_SNOOZE_TIME(self) -> None: ...

    @property
    def X_MOZ_LASTACK(self) -> datetime.datetime | None: ...
    @X_MOZ_LASTACK.setter
    def X_MOZ_LASTACK(self, value: datetime.datetime) -> None: ...
    @X_MOZ_LASTACK.deleter
    def X_MOZ_LASTACK(self) -> None: ...

    @property
    def color(self) -> str: ...
    @color.setter
    def color(self, value: str) -> None: ...
    @color.deleter
    def color(self) -> None: ...

    @property
    def sequence(self) -> int: ...
    @sequence.setter
    def sequence(self, value: int) -> None: ...
    @sequence.deleter
    def sequence(self) -> None: ...

    @property
    def categories(self) -> list[str]: ...
    @categories.setter
    def categories(self, cats: list[str]) -> None: ...
    @categories.deleter
    def categories(self) -> None: ...

    @property
    def rdates(
        self,
    ) -> list[tuple[datetime.date, None] | tuple[datetime.datetime, None] | tuple[datetime.datetime, datetime.datetime]]: ...
    @property
    def exdates(self) -> list[datetime.date | datetime.datetime]: ...
    @property
    def rrules(self) -> list[vRecur]: ...

    @property
    def uid(self) -> str: ...
    @uid.setter
    def uid(self, value: str) -> None: ...
    @uid.deleter
    def uid(self) -> None: ...

class Todo(Component):
    name: ClassVar[Literal["VTODO"]]

    @property
    def DTSTART(self) -> datetime.datetime | datetime.date | None: ...
    @DTSTART.setter
    def DTSTART(self, value: datetime.datetime | datetime.date | None) -> None: ...
    @DTSTART.deleter
    def DTSTART(self) -> None: ...

    @property
    def DUE(self) -> datetime.datetime | datetime.date | None: ...
    @DUE.setter
    def DUE(self, value: datetime.datetime | datetime.date | None) -> None: ...
    @DUE.deleter
    def DUE(self) -> None: ...

    @property
    def DURATION(self) -> datetime.timedelta | None: ...
    @DURATION.setter
    def DURATION(self, value: datetime.timedelta | None) -> None: ...
    @DURATION.deleter
    def DURATION(self) -> None: ...

    @property
    def start(self) -> datetime.datetime | datetime.date: ...
    @start.setter
    def start(self, value: datetime.datetime | datetime.date | None) -> None: ...

    @property
    def end(self) -> datetime.datetime | datetime.date: ...
    @end.setter
    def end(self, value: datetime.datetime | datetime.date | None) -> None: ...

    @property
    def duration(self) -> datetime.timedelta: ...

    @property
    def X_MOZ_SNOOZE_TIME(self) -> datetime.datetime | None: ...
    @X_MOZ_SNOOZE_TIME.setter
    def X_MOZ_SNOOZE_TIME(self, value: datetime.datetime) -> None: ...
    @X_MOZ_SNOOZE_TIME.deleter
    def X_MOZ_SNOOZE_TIME(self) -> None: ...

    @property
    def X_MOZ_LASTACK(self) -> datetime.datetime | None: ...
    @X_MOZ_LASTACK.setter
    def X_MOZ_LASTACK(self, value: datetime.datetime) -> None: ...
    @X_MOZ_LASTACK.deleter
    def X_MOZ_LASTACK(self) -> None: ...

    @property
    def alarms(self) -> Alarms: ...

    @property
    def color(self) -> str: ...
    @color.setter
    def color(self, value: str) -> None: ...
    @color.deleter
    def color(self) -> None: ...

    @property
    def sequence(self) -> int: ...
    @sequence.setter
    def sequence(self, value: int) -> None: ...
    @sequence.deleter
    def sequence(self) -> None: ...

    @property
    def categories(self) -> list[str]: ...
    @categories.setter
    def categories(self, cats: list[str]) -> None: ...
    @categories.deleter
    def categories(self) -> None: ...

    @property
    def rdates(
        self,
    ) -> list[tuple[datetime.date, None] | tuple[datetime.datetime, None] | tuple[datetime.datetime, datetime.datetime]]: ...
    @property
    def exdates(self) -> list[datetime.date | datetime.datetime]: ...
    @property
    def rrules(self) -> list[vRecur]: ...

    @property
    def uid(self) -> str: ...
    @uid.setter
    def uid(self, value: str) -> None: ...
    @uid.deleter
    def uid(self) -> None: ...

class Journal(Component):
    name: ClassVar[Literal["VJOURNAL"]]

    @property
    def DTSTART(self) -> datetime.date | datetime.datetime | None: ...
    @DTSTART.setter
    def DTSTART(self, value: datetime.date | datetime.datetime | None) -> None: ...
    @DTSTART.deleter
    def DTSTART(self) -> None: ...

    @property
    def start(self) -> datetime.date | datetime.datetime: ...
    @start.setter
    def start(self, value: datetime.date | datetime.datetime | None) -> None: ...

    end = start
    @property
    def duration(self) -> datetime.timedelta: ...

    @property
    def color(self) -> str: ...
    @color.setter
    def color(self, value: str) -> None: ...
    @color.deleter
    def color(self) -> None: ...

    @property
    def sequence(self) -> int: ...
    @sequence.setter
    def sequence(self, value: int) -> None: ...
    @sequence.deleter
    def sequence(self) -> None: ...

    @property
    def categories(self) -> list[str]: ...
    @categories.setter
    def categories(self, cats: list[str]) -> None: ...
    @categories.deleter
    def categories(self) -> None: ...

    @property
    def rdates(
        self,
    ) -> list[tuple[datetime.date, None] | tuple[datetime.datetime, None] | tuple[datetime.datetime, datetime.datetime]]: ...
    @property
    def exdates(self) -> list[datetime.date | datetime.datetime]: ...
    @property
    def rrules(self) -> list[vRecur]: ...

    @property
    def uid(self) -> str: ...
    @uid.setter
    def uid(self, value: str) -> None: ...
    @uid.deleter
    def uid(self) -> None: ...

class FreeBusy(Component):
    name: ClassVar[Literal["VFREEBUSY"]]

    @property
    def uid(self) -> str: ...
    @uid.setter
    def uid(self, value: str) -> None: ...
    @uid.deleter
    def uid(self) -> None: ...

class Timezone(Component):
    subcomponents: list[TimezoneStandard | TimezoneDaylight]
    name: ClassVar[Literal["VTIMEZONE"]]
    @classmethod
    def example(cls, name: str = "pacific_fiji") -> Calendar: ...
    def to_tz(self, tzp: TZP = ..., lookup_tzid: bool = True) -> datetime.tzinfo: ...
    @property
    def tz_name(self) -> str: ...
    def get_transitions(self) -> tuple[list[datetime.datetime], list[tuple[datetime.timedelta, datetime.timedelta, str]]]: ...
    @classmethod
    def from_tzinfo(
        cls, timezone: datetime.tzinfo, tzid: str | None = None, first_date: datetime.date = ..., last_date: datetime.date = ...
    ) -> Self: ...
    @classmethod
    def from_tzid(cls, tzid: str, tzp: TZP = ..., first_date: datetime.date = ..., last_date: datetime.date = ...) -> Self: ...
    @property
    def standard(self) -> list[TimezoneStandard]: ...
    @property
    def daylight(self) -> list[TimezoneDaylight]: ...

class TimezoneStandard(Component):
    name: ClassVar[Literal["STANDARD"]]

    @property
    def DTSTART(self) -> datetime.date | datetime.datetime | None: ...
    @DTSTART.setter
    def DTSTART(self, value: datetime.date | datetime.datetime | None) -> None: ...
    @DTSTART.deleter
    def DTSTART(self) -> None: ...

    @property
    def TZOFFSETTO(self) -> datetime.timedelta | None: ...
    @TZOFFSETTO.setter
    def TZOFFSETTO(self, value: datetime.timedelta | None) -> None: ...
    @TZOFFSETTO.deleter
    def TZOFFSETTO(self) -> None: ...

    @property
    def TZOFFSETFROM(self) -> datetime.timedelta | None: ...
    @TZOFFSETFROM.setter
    def TZOFFSETFROM(self, value: datetime.timedelta | None) -> None: ...
    @TZOFFSETFROM.deleter
    def TZOFFSETFROM(self) -> None: ...

    @property
    def rdates(
        self,
    ) -> list[tuple[datetime.date, None] | tuple[datetime.datetime, None] | tuple[datetime.datetime, datetime.datetime]]: ...
    @property
    def exdates(self) -> list[datetime.date | datetime.datetime]: ...
    @property
    def rrules(self) -> list[vRecur]: ...

class TimezoneDaylight(Component):
    name: ClassVar[Literal["DAYLIGHT"]]

    @property
    def DTSTART(self) -> datetime.date | datetime.datetime | None: ...
    @DTSTART.setter
    def DTSTART(self, value: datetime.date | datetime.datetime | None) -> None: ...
    @DTSTART.deleter
    def DTSTART(self) -> None: ...

    @property
    def TZOFFSETTO(self) -> datetime.timedelta | None: ...
    @TZOFFSETTO.setter
    def TZOFFSETTO(self, value: datetime.timedelta | None) -> None: ...
    @TZOFFSETTO.deleter
    def TZOFFSETTO(self) -> None: ...

    @property
    def TZOFFSETFROM(self) -> datetime.timedelta | None: ...
    @TZOFFSETFROM.setter
    def TZOFFSETFROM(self, value: datetime.timedelta | None) -> None: ...
    @TZOFFSETFROM.deleter
    def TZOFFSETFROM(self) -> None: ...

    @property
    def rdates(
        self,
    ) -> list[tuple[datetime.date, None] | tuple[datetime.datetime, None] | tuple[datetime.datetime, datetime.datetime]]: ...
    @property
    def exdates(self) -> list[datetime.date | datetime.datetime]: ...
    @property
    def rrules(self) -> list[vRecur]: ...

class Alarm(Component):
    name: ClassVar[Literal["VALARM"]]

    @property
    def REPEAT(self) -> int: ...
    @REPEAT.setter
    def REPEAT(self, value: int) -> None: ...
    @REPEAT.deleter
    def REPEAT(self) -> None: ...

    @property
    def DURATION(self) -> datetime.timedelta | None: ...
    @DURATION.setter
    def DURATION(self, value: datetime.timedelta | None) -> None: ...
    @DURATION.deleter
    def DURATION(self) -> None: ...

    @property
    def ACKNOWLEDGED(self) -> datetime.datetime | None: ...
    @ACKNOWLEDGED.setter
    def ACKNOWLEDGED(self, value: datetime.datetime | None) -> None: ...
    @ACKNOWLEDGED.deleter
    def ACKNOWLEDGED(self) -> None: ...

    @property
    def TRIGGER(self) -> datetime.timedelta | datetime.datetime | None: ...
    @TRIGGER.setter
    def TRIGGER(self, value: datetime.timedelta | datetime.datetime | None) -> None: ...
    @TRIGGER.deleter
    def TRIGGER(self) -> None: ...

    @property
    def TRIGGER_RELATED(self) -> Literal["START", "END"]: ...
    @TRIGGER_RELATED.setter
    def TRIGGER_RELATED(self, value: Literal["START", "END"]) -> None: ...

    class Triggers(NamedTuple):
        start: tuple[datetime.timedelta, ...]
        end: tuple[datetime.timedelta, ...]
        absolute: tuple[datetime.datetime, ...]

    @property
    def triggers(self) -> Alarm.Triggers: ...

    @property
    def uid(self) -> str: ...
    @uid.setter
    def uid(self, value: str) -> None: ...
    @uid.deleter
    def uid(self) -> None: ...

class Calendar(Component):
    name: ClassVar[Literal["VCALENDAR"]]
    @classmethod
    def example(cls, name: str = "example") -> Calendar: ...
    @property
    def freebusy(self) -> list[FreeBusy]: ...
    @property
    def events(self) -> list[Event]: ...
    @property
    def todos(self) -> list[Todo]: ...
    def get_used_tzids(self) -> set[str]: ...
    def get_missing_tzids(self) -> set[str]: ...
    @property
    def timezones(self) -> list[Timezone]: ...
    def add_missing_timezones(self, first_date: datetime.date = ..., last_date: datetime.date = ...) -> None: ...

    @property
    def calendar_name(self) -> str | None: ...
    @calendar_name.setter
    def calendar_name(self, value: str) -> None: ...
    @calendar_name.deleter
    def calendar_name(self) -> None: ...

    @property
    def description(self) -> str | None: ...
    @description.setter
    def description(self, value: str) -> None: ...
    @description.deleter
    def description(self) -> None: ...

    @property
    def color(self) -> str: ...
    @color.setter
    def color(self, value: str) -> None: ...
    @color.deleter
    def color(self) -> None: ...

    @property
    def categories(self) -> list[str]: ...
    @categories.setter
    def categories(self, cats: list[str]) -> None: ...
    @categories.deleter
    def categories(self) -> None: ...

    @property
    def uid(self) -> str: ...
    @uid.setter
    def uid(self, value: str) -> None: ...
    @uid.deleter
    def uid(self) -> None: ...

types_factory: Final[TypesFactory]
component_factory: Final[ComponentFactory]



import abc
import pathlib
import sys
import types
from _collections_abc import dict_keys, dict_values
from _typeshed import StrPath
from collections.abc import Iterable, Iterator, Mapping
from importlib.abc import MetaPathFinder
from importlib.metadata._meta import PackageMetadata as PackageMetadata, SimplePath
from os import PathLike
from re import Pattern
from typing import Any, ClassVar, Generic, NamedTuple, TypeAlias, TypeVar, overload, type_check_only
from typing_extensions import Self, deprecated, disjoint_base

_T = TypeVar("_T")
_KT = TypeVar("_KT")
_VT = TypeVar("_VT")

__all__ = [
    "Distribution",
    "DistributionFinder",
    "PackageMetadata",
    "PackageNotFoundError",
    "distribution",
    "distributions",
    "entry_points",
    "files",
    "metadata",
    "packages_distributions",
    "requires",
    "version",
]

if sys.version_info >= (3, 15):
    __all__ += ["PackagePath", "MetadataNotFound", "SimplePath"]

_SimplePath: TypeAlias = SimplePath

def packages_distributions() -> Mapping[str, list[str]]: ...

class PackageNotFoundError(ModuleNotFoundError):
    @property
    def name(self) -> str: ...  # type: ignore[override]

if sys.version_info >= (3, 15):
    class MetadataNotFound(FileNotFoundError): ...

if sys.version_info >= (3, 13):
    _EntryPointBase = object
elif sys.version_info >= (3, 11):
    class DeprecatedTuple:
        def __getitem__(self, item: int) -> str: ...

    _EntryPointBase = DeprecatedTuple
else:
    @type_check_only
    class _EntryPointBase(NamedTuple):
        name: str
        value: str
        group: str

if sys.version_info >= (3, 11):
    class EntryPoint(_EntryPointBase):
        pattern: ClassVar[Pattern[str]]
        name: str
        value: str
        group: str

        def __init__(self, name: str, value: str, group: str) -> None: ...
        def load(self) -> Any: ...  # Callable[[], Any] or an importable module
        @property
        def extras(self) -> list[str]: ...
        @property
        def module(self) -> str: ...
        @property
        def attr(self) -> str: ...
        dist: ClassVar[Distribution | None]
        def matches(
            self,
            *,
            name: str = ...,
            value: str = ...,
            group: str = ...,
            module: str = ...,
            attr: str = ...,
            extras: list[str] = ...,
        ) -> bool: ...  # undocumented
        def __hash__(self) -> int: ...
        def __eq__(self, other: object) -> bool: ...
        def __lt__(self, other: object) -> bool: ...
        if sys.version_info < (3, 12):
            def __iter__(self) -> Iterator[Any]: ...  # result of iter((str, Self)), really

else:
    @disjoint_base
    class EntryPoint(_EntryPointBase):
        pattern: ClassVar[Pattern[str]]

        def load(self) -> Any: ...  # Callable[[], Any] or an importable module
        @property
        def extras(self) -> list[str]: ...
        @property
        def module(self) -> str: ...
        @property
        def attr(self) -> str: ...
        dist: ClassVar[Distribution | None]
        def matches(
            self,
            *,
            name: str = ...,
            value: str = ...,
            group: str = ...,
            module: str = ...,
            attr: str = ...,
            extras: list[str] = ...,
        ) -> bool: ...  # undocumented
        def __hash__(self) -> int: ...
        def __iter__(self) -> Iterator[Any]: ...  # result of iter((str, Self)), really

if sys.version_info >= (3, 12):
    class EntryPoints(tuple[EntryPoint, ...]):
        __slots__ = ()
        def __getitem__(self, name: str) -> EntryPoint: ...  # type: ignore[override]
        def select(
            self,
            *,
            name: str = ...,
            value: str = ...,
            group: str = ...,
            module: str = ...,
            attr: str = ...,
            extras: list[str] = ...,
        ) -> EntryPoints: ...
        @property
        def names(self) -> set[str]: ...
        @property
        def groups(self) -> set[str]: ...

else:
    class DeprecatedList(list[_T]):
        __slots__ = ()

    class EntryPoints(DeprecatedList[EntryPoint]):  # use as list is deprecated since 3.10
        # int argument is deprecated since 3.10
        __slots__ = ()
        def __getitem__(self, name: int | str) -> EntryPoint: ...  # type: ignore[override]
        def select(
            self,
            *,
            name: str = ...,
            value: str = ...,
            group: str = ...,
            module: str = ...,
            attr: str = ...,
            extras: list[str] = ...,
        ) -> EntryPoints: ...
        @property
        def names(self) -> set[str]: ...
        @property
        def groups(self) -> set[str]: ...

if sys.version_info < (3, 12):
    class Deprecated(Generic[_KT, _VT]):
        def __getitem__(self, name: _KT) -> _VT: ...

        @overload
        def get(self, name: _KT, default: None = None) -> _VT | None: ...
        @overload
        def get(self, name: _KT, default: _VT) -> _VT: ...
        @overload
        def get(self, name: _KT, default: _T) -> _VT | _T: ...

        def __iter__(self) -> Iterator[_KT]: ...
        def __contains__(self, *args: object) -> bool: ...
        def keys(self) -> dict_keys[_KT, _VT]: ...
        def values(self) -> dict_values[_KT, _VT]: ...

    @deprecated("Deprecated since Python 3.10; removed in Python 3.12. Use `select` instead.")
    class SelectableGroups(Deprecated[str, EntryPoints], dict[str, EntryPoints]):  # use as dict is deprecated since 3.10
        @classmethod
        def load(cls, eps: Iterable[EntryPoint]) -> Self: ...
        @property
        def groups(self) -> set[str]: ...
        @property
        def names(self) -> set[str]: ...

        @overload
        def select(self) -> Self: ...
        @overload
        def select(
            self,
            *,
            name: str = ...,
            value: str = ...,
            group: str = ...,
            module: str = ...,
            attr: str = ...,
            extras: list[str] = ...,
        ) -> EntryPoints: ...

class PackagePath(pathlib.PurePosixPath):
    def read_text(self, encoding: str = "utf-8") -> str: ...
    def read_binary(self) -> bytes: ...
    def locate(self) -> PathLike[str]: ...
    # The following attributes are not defined on PackagePath, but are dynamically added by Distribution.files:
    hash: FileHash | None
    size: int | None
    dist: Distribution

class FileHash:
    mode: str
    value: str
    def __init__(self, spec: str) -> None: ...

if sys.version_info >= (3, 15):
    _distribution_parent = abc.ABC
elif sys.version_info >= (3, 12):
    class DeprecatedNonAbstract: ...
    _distribution_parent = DeprecatedNonAbstract
else:
    _distribution_parent = object

class Distribution(_distribution_parent):
    @abc.abstractmethod
    def read_text(self, filename: str) -> str | None: ...
    @abc.abstractmethod
    def locate_file(self, path: StrPath) -> _SimplePath: ...
    @classmethod
    def from_name(cls, name: str) -> Distribution: ...

    @overload
    @classmethod
    def discover(cls, *, context: DistributionFinder.Context) -> Iterable[Distribution]: ...
    @overload
    @classmethod
    def discover(
        cls, *, context: None = None, name: str | None = ..., path: list[str] = ..., **kwargs: Any
    ) -> Iterable[Distribution]: ...

    @staticmethod
    def at(path: StrPath) -> PathDistribution: ...
    @property
    def metadata(self) -> PackageMetadata: ...
    @property
    def entry_points(self) -> EntryPoints: ...
    @property
    def version(self) -> str: ...
    @property
    def files(self) -> list[PackagePath] | None: ...
    @property
    def requires(self) -> list[str] | None: ...
    @property
    def name(self) -> str: ...
    if sys.version_info >= (3, 13):
        @property
        def origin(self) -> types.SimpleNamespace | None: ...

class DistributionFinder(MetaPathFinder):
    class Context:
        name: str | None
        def __init__(self, *, name: str | None = ..., path: list[str] = ..., **kwargs: Any) -> None: ...
        @property
        def path(self) -> list[str]: ...

    @abc.abstractmethod
    def find_distributions(self, context: DistributionFinder.Context = ...) -> Iterable[Distribution]: ...

class MetadataPathFinder(DistributionFinder):
    @classmethod
    def find_distributions(cls, context: DistributionFinder.Context = ...) -> Iterable[PathDistribution]: ...
    if sys.version_info >= (3, 11):
        @classmethod
        def invalidate_caches(cls) -> None: ...
    else:
        # Yes, this is an instance method that has a parameter named "cls"
        def invalidate_caches(cls) -> None: ...

class PathDistribution(Distribution):
    _path: _SimplePath
    def __init__(self, path: _SimplePath) -> None: ...
    def read_text(self, filename: StrPath) -> str | None: ...
    def locate_file(self, path: StrPath) -> _SimplePath: ...

def distribution(distribution_name: str) -> Distribution: ...

@overload
def distributions(*, context: DistributionFinder.Context) -> Iterable[Distribution]: ...
@overload
def distributions(
    *, context: None = None, name: str | None = ..., path: list[str] = ..., **kwargs: Any
) -> Iterable[Distribution]: ...

def metadata(distribution_name: str) -> PackageMetadata: ...

if sys.version_info >= (3, 12):
    def entry_points(
        *, name: str = ..., value: str = ..., group: str = ..., module: str = ..., attr: str = ..., extras: list[str] = ...
    ) -> EntryPoints: ...

else:
    @overload
    def entry_points() -> SelectableGroups: ...
    @overload
    def entry_points(
        *, name: str = ..., value: str = ..., group: str = ..., module: str = ..., attr: str = ..., extras: list[str] = ...
    ) -> EntryPoints: ...

def version(distribution_name: str) -> str: ...
def files(distribution_name: str) -> list[PackagePath] | None: ...
def requires(distribution_name: str) -> list[str] | None: ...



from re import Match, Pattern, RegexFlag

class EndOfText(RuntimeError): ...

class Scanner:
    data: str
    data_length: int
    start_pos: int
    pos: int
    flags: int | RegexFlag
    last: str | None
    match: str | None
    def __init__(self, text: str, flags: int | RegexFlag = 0) -> None: ...
    @property
    def eos(self) -> bool: ...
    def check(self, pattern: str | Pattern[str]) -> Match[str] | None: ...
    def test(self, pattern: str | Pattern[str]) -> bool: ...
    def scan(self, pattern: str | Pattern[str]) -> bool: ...
    def get_char(self) -> None: ...



from _typeshed import Incomplete
from collections.abc import Iterable

from networkx.algorithms.shortest_paths.weighted import _WeightFunc
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from numpy.random import RandomState

__all__ = ["kernighan_lin_bisection", "spectral_modularity_bipartition", "greedy_node_swap_bipartition"]

@_dispatchable
def kernighan_lin_bisection(
    G: Graph[_Node],
    partition: tuple[Iterable[Incomplete], Iterable[Incomplete]] | None = None,
    max_iter: int = 10,
    weight: str | _WeightFunc[_Node] = "weight",
    seed: int | RandomState | None = None,
) -> tuple[set[Incomplete], set[Incomplete]]: ...
def spectral_modularity_bipartition(G: Graph[_Node]) -> tuple[set[Incomplete], set[Incomplete]]: ...
def greedy_node_swap_bipartition(
    G: Graph[_Node], *, init_split: tuple[set[Incomplete], set[Incomplete]] | None = None, max_iter: int = 10
) -> tuple[set[Incomplete], set[Incomplete]]: ...



from collections.abc import Mapping, Sequence

__all__ = ["readmodule", "readmodule_ex", "Class", "Function"]

class _Object:
    module: str
    name: str
    file: int
    lineno: int
    end_lineno: int | None
    parent: _Object | None

    # This is a dict at runtime, but we're typing it as Mapping to
    # avoid variance issues in the subclasses
    children: Mapping[str, _Object]

    def __init__(
        self, module: str, name: str, file: str, lineno: int, end_lineno: int | None, parent: _Object | None
    ) -> None: ...

class Function(_Object):
    is_async: bool
    parent: Function | Class | None
    children: dict[str, Class | Function]

    def __init__(
        self,
        module: str,
        name: str,
        file: str,
        lineno: int,
        parent: Function | Class | None = None,
        is_async: bool = False,
        *,
        end_lineno: int | None = None,
    ) -> None: ...

class Class(_Object):
    super: list[Class | str] | None
    methods: dict[str, int]
    parent: Class | None
    children: dict[str, Class | Function]

    def __init__(
        self,
        module: str,
        name: str,
        super_: list[Class | str] | None,
        file: str,
        lineno: int,
        parent: Class | None = None,
        *,
        end_lineno: int | None = None,
    ) -> None: ...

def readmodule(module: str, path: Sequence[str] | None = None) -> dict[str, Class]: ...
def readmodule_ex(module: str, path: Sequence[str] | None = None) -> dict[str, Class | Function | list[str]]: ...



import re
from _typeshed import Incomplete
from datetime import datetime
from typing import Final
from zoneinfo import ZoneInfo

from dateparser.conf import Settings
from dateparser.date import DateData

PATTERN: Final[re.Pattern[str]]

class FreshnessDateDataParser:
    def get_local_tz(self) -> ZoneInfo: ...
    def parse(self, date_string: str, settings: Settings) -> tuple[datetime | None, str | None]: ...
    def get_kwargs(
        self, date_string: str
    ) -> tuple[dict[str, float], dict[str, Incomplete]] | dict[None, None]: ...  # return empty dict if pattern not found
    def get_date_data(self, date_string: str, settings: Settings | None = None) -> DateData: ...

freshness_date_parser: FreshnessDateDataParser



from _typeshed import Incomplete
from collections.abc import Callable, Iterable
from typing import Literal, SupportsIndex, TypeAlias

import pythoncom

com_error = pythoncom.com_error

# Type of pythoncom._univgw.WriteFromOutTuple
# The two tuples must be of equal length
_WriteFromOutTupleType: TypeAlias = Callable[
    [tuple[Incomplete, ...] | None, tuple[Incomplete, ...] | None, int], Incomplete | None
]

def RegisterInterfaces(
    typelibGUID, lcid, major, minor, interface_names: Iterable[str] | None = None
) -> list[tuple[Incomplete, Incomplete, Incomplete]]: ...

class Arg:
    name: Incomplete
    vt: Incomplete
    inOut: Incomplete
    default: Incomplete
    clsid: Incomplete
    size: Incomplete
    offset: int
    def __init__(self, arg_info, name=None) -> None: ...

class Method:
    dispid: Incomplete
    invkind: Incomplete
    name: Incomplete
    args: list[Arg]
    cbArgs: Incomplete
    def __init__(self, method_info, isEventSink: bool | Literal[0, 1] = 0) -> None: ...

class Definition:
    def __init__(self, iid, is_dispatch, method_defs) -> None: ...
    def iid(self): ...
    def vtbl_argsizes(self) -> list[Incomplete]: ...
    def vtbl_argcounts(self) -> list[int]: ...
    def dispatch(
        self, ob, index: SupportsIndex, argPtr, ReadFromInTuple=..., WriteFromOutTuple: _WriteFromOutTupleType = ...
    ): ...



import socket
from _typeshed import Unused
from collections import deque
from collections.abc import Callable
from concurrent.futures import Future, ThreadPoolExecutor
from selectors import DefaultSelector
from types import FrameType
from typing import Final

from gunicorn.config import Config
from gunicorn.glogging import Logger as GLogger
from gunicorn.http import Request, RequestParser
from gunicorn.http2.connection import HTTP2ServerConnection
from gunicorn.uwsgi.parser import UWSGIParser

from .._types import _AddressType
from . import base

DEFAULT_WORKER_DATA_TIMEOUT: Final = 5.0

class TConn:
    cfg: Config
    sock: socket.socket
    client: _AddressType
    server: _AddressType
    timeout: float | None
    parser: HTTP2ServerConnection | UWSGIParser | RequestParser | None
    initialized: bool
    is_http2: bool
    data_ready: bool

    def __init__(self, cfg: Config, sock: socket.socket, client: _AddressType, server: _AddressType) -> None: ...
    def init(self) -> None: ...
    def set_timeout(self) -> None: ...
    def wait_for_data(self, timeout: float | None) -> bool: ...
    def close(self, graceful: bool = False) -> None: ...

class PollableMethodQueue:
    def __init__(self) -> None: ...
    def init(self) -> None: ...
    def close(self) -> None: ...
    def fileno(self) -> int | None: ...
    # Actually, `*args` are expected to match the parameter types of `callback`.
    # Ideally this would be typed using ParamSpec as:
    # def defer(self, callback: Callable[_P, object], *args: _P.args) -> None
    def defer(self, callback: Callable[..., object], *args: object) -> None: ...
    def run_callbacks(self, _fileobj: Unused, max_callbacks: int = 50) -> None: ...

class ThreadWorker(base.Worker):
    worker_connections: int
    max_keepalived: int
    tpool: ThreadPoolExecutor
    poller: DefaultSelector
    method_queue: PollableMethodQueue
    keepalived_conns: deque[TConn]
    pending_conns: deque[TConn]
    nr_conns: int
    alive: bool

    @classmethod
    def check_config(cls, cfg: Config, log: GLogger) -> None: ...
    def init_process(self) -> None: ...
    def get_thread_pool(self) -> ThreadPoolExecutor: ...
    def handle_exit(self, sig: int, frame: FrameType | None) -> None: ...
    def handle_quit(self, sig: int, frame: FrameType | None) -> None: ...
    def set_accept_enabled(self, enabled: bool | None) -> None: ...
    def enqueue_req(self, conn: TConn) -> None: ...
    def accept(self, listener: socket.socket) -> None: ...
    def on_client_socket_readable(self, conn: TConn, client: socket.socket) -> None: ...
    def on_pending_socket_readable(self, conn: TConn, client: socket.socket) -> None: ...
    def murder_keepalived(self) -> None: ...
    def murder_pending(self) -> None: ...
    def is_parent_alive(self) -> bool: ...
    def wait_for_and_dispatch_events(self, timeout: float | None) -> None: ...
    def run(self) -> None: ...
    def finish_request(self, conn: TConn, fs: Future[bool]) -> None: ...
    def handle(self, conn: TConn) -> bool: ...
    def handle_http2(self, conn: TConn) -> bool: ...
    def handle_http2_request(self, req: Request, conn: TConn, h2_conn: HTTP2ServerConnection) -> None: ...
    def handle_request(self, req: Request, conn: TConn) -> bool: ...



from _typeshed import Incomplete
from typing import Literal
from typing_extensions import LiteralString

class Formatter:
    pattern: str
    def __init__(self, pattern: str) -> None: ...
    def format(self, obj: object) -> str: ...
    def __call__(self, x: object) -> str: ...

class DecimalFormatter(Formatter):
    calcPlaces: Incomplete
    places: int
    dot: Incomplete
    comma: Incomplete
    prefix: Incomplete
    suffix: Incomplete
    def __init__(
        self, places: int | Literal["auto"] = 2, decimalSep: str = ".", thousandSep=None, prefix=None, suffix=None
    ) -> None: ...
    def format(self, num) -> LiteralString: ...

__all__ = ("Formatter", "DecimalFormatter")
